% Solving the Poisson equation by Q5 SEM with Neumann b.c.
if gpuDeviceCount('available')<1; Param.device='cpu';
else;Param.device='gpu'; Param.deviceID=1;end % ID=1,2,3,...
Np=5; Param.Np=Np; % polynomial degree Q5
Ncellx=40; Ncelly=40; Ncellz=40; % finite element cell number
% total number of unknowns in each direction
nx=Ncellx*Np+1; ny=Ncelly*Np+1; nz=Ncellz*Np+1;
% the domain is [-Lx, Lx]*[-Ly, Ly]*[-Lz, Lz]
Lx=1; Ly=1; Lz=1; cx=pi; cy=2*pi; cz=3*pi; alpha=1;
Param.Ncellx=Ncellx; Param.Ncelly=Ncelly; Param.Ncellz=Ncellz;
Param.nx = nx; Param.ny = ny; Param.nz = nz;
fprintf('3D Poisson with total DoFs %d by %d by %d \n',nx,ny,nz);
fprintf('Laplacian is Q%d spectral element method \n', Np);  
[x,ex,Tx,eigx]=SEGenerator1D('x',Lx,Param);
[y,ey,Ty,eigy]=SEGenerator1D('y',Ly,Param);
[z,ez,Tz,eigz]=SEGenerator1D('z',Lz,Param);
% a smooth solution
u1x=cos(cx*x); u2x=power(1-power(x,2),3); 
du2x=30*power(x,4)-36*power(x,2)+6;
u1y=cos(cy*y); u2y=power(1-power(y,2),2); 
du2y=4-12*power(y,2);
u1z=cos(cz*z); u2z=power(1-power(z,2),4); 
du2z=(8-56*power(z,2)).*power(1-power(z,2),2);
uexact=squeeze(tensorprod(u1x*u1y',u1z)+tensorprod(u2x*u2y',u2z));
f=(cx*cx+cy*cy+cz*cz)*squeeze(tensorprod(u1x*u1y',u1z))+...
  squeeze(tensorprod(du2x*u2y',u2z)+tensorprod(u2x*du2y',u2z)+...
  tensorprod(u2x*u2y',du2z))+alpha*uexact;
TxInv=pinv(Tx); TyInv=pinv(Ty); TzInv=pinv(Tz);
if strcmp(Param.device,'gpu');Device=gpuDevice(Param.deviceID);
   fprintf('GPU computation: starting to load matrices/data \n');
   Tx=gpuArray(Tx); Ty=gpuArray(Ty); Tz=gpuArray(Tz);
   eigx=gpuArray(eigx);eigy=gpuArray(eigy);eigz=gpuArray(eigz);
   ex=gpuArray(ex); ey=gpuArray(ey); ez=gpuArray(ez); f=gpuArray(f); 
   TxInv=gpuArray(TxInv);TyInv=gpuArray(TyInv);TzInv=gpuArray(TzInv);
end 
Lambda3D=squeeze(tensorprod(eigx,ey*ez')+tensorprod(ex,eigy*ez')...
        +tensorprod(ex,ey*eigz'));
if strcmp(Param.device,'gpu'); wait(Device);
    fprintf('GPU loading finished and computing started \n');
end
tic; % online computation
u = tensorprod(f,TzInv',3,1);
u = pagemtimes(u,TyInv');
u = squeeze(tensorprod(TxInv,u,2,1));
u = u./(Lambda3D + alpha);
u = tensorprod(u,Tz',3,1);
u = pagemtimes(u,Ty');
u = squeeze(tensorprod(Tx,u,2,1));
if strcmp(Param.device,'gpu');wait(Device);end;time=toc;err=u-uexact;
fprintf('The ell infinity norm error is %d \n',norm(err(:),inf));
if strcmp(Param.device,'gpu')
   fprintf('The online GPU computation time is %d \n', time);
else
   fprintf('The online CPU computation time is %d \n', time);
end
